(* lgd file reader *)
IMPLEMENTATION MODULE Rdr;

FROM SYSTEM IMPORT BYTE;
IMPORT IO, FIO, FIOStd, Str;
IMPORT Lib2, Lib3, SO, Config, FNames;

CONST
  srcNull ::= FIOStd.inv;

VAR
  src: FIO.File;


PROCEDURE Error( x-: ARRAY OF CHAR );
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;
BEGIN
  SO.Init( b, s );
  SO.WrStr( b, 'Rdr.Err: ' );
  SO.WrStr( b, x );
  SO.WrStr( b, ', pos ' );
  SO.WrLngHex( b, pos, 1 );
  SO.WrStr( b, ' top ' );
  SO.WrLngHex( b, top, 1 );
  IO.WrStr( s );
  IO.WrLn();
END Error;

PROCEDURE Message( x-: ARRAY OF CHAR );
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;
BEGIN
  SO.Init( b, s );
  SO.WrStr( b, 'Rdr.Msg: ' );
  SO.WrStr( b, x );
  IO.WrStr( s );
  IO.WrLn();
END Message;


PROCEDURE FRead( VAR x: ARRAY OF BYTE; l: CARDINAL ): CARDINAL;
BEGIN  (* local buffer ??? *)
  IF l > SIZE( x ) THEN l := SIZE( x ) END;
  RETURN FIO.RdBin( src, x, l );
END FRead;


PROCEDURE Open();

  PROCEDURE Show();
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    IF ~ Config.data.vrb THEN RETURN END;
    SO.Init( b, s );
    SO.WrStr( b, 'Open: opn ' );
    SO.WrLngCard( b, opn, 1 );
    SO.WrStr( b, ' src "' );
    SO.WrStr( b, FNames.in );
    SO.WrStr( b, '" fmt ' );
    SO.WrShtCard( b, fmt, 1 );
    Message( s );
  END Show;

BEGIN
  Close();
  IF ~ FIO.Exists( FNames.in ) THEN
    Error( 'Open: files does not exist' );
    Show();
    RETURN;
  END;
  src := FIO.OpenRead( FNames.in );
  IF src = srcNull THEN
    Error( 'Open: cannot open file' );
    Show();
    RETURN;
  END;
  fdt := FIO.GetFileDate( src );
  IF FRead( fmt, SIZE(fmt) ) # SIZE( fmt ) THEN
    Error( 'Open: cannot read format' );
    Show();
    RETURN;
  END;
  top := SIZE( fmt );
  ok := ( fmt = 3 ) OR ( fmt = 7 ) OR ( fmt = 8 );
  IF ~ ok THEN
    Error( 'Open: unsupported file format' );
    Show();
    RETURN;
  END;
  INC( opn );
  Show();
END Open;

PROCEDURE Read();
VAR
  l: CARDINAL;
BEGIN
  ok := FALSE;
  IF src = srcNull THEN RETURN END;
  pos := top;
  (* { len ( tel | ref tel ) } *)
  (* len *)
  l := FRead( len, SIZE(len) );
  IF l = 0 THEN (* eof *) RETURN END;
  IF l # SIZE( len ) THEN Error( 'Read: cannot read tel len' ); RETURN END;
  INC( top, l );
  IF fmt = 7 THEN
    (* ref *)
    IF len < SIZE( ref ) THEN Error( 'Read: tel len too small' ); RETURN END;
    DEC( len, SIZE(ref) );
    l := FRead( ref, SIZE(ref) );
    IF l # SIZE( ref ) THEN Error( 'Read: cannot read ref' ); RETURN END;
    INC( top, l );
  END;
  IF len = 0 THEN Error( 'Read: tel len is zero' ); RETURN END;
  IF len > SIZE( tel ) THEN Error( 'Read: tel len too large' ); RETURN END;
  (* tel *)
  l := FRead( tel, len );
  IF l # len THEN Error( 'Read: cannot read tel' ); RETURN END;
  INC( top, l );
  INC( cnt );
  ok := TRUE;
END Read;

PROCEDURE Close();

  PROCEDURE Show();
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    IF ~ Config.data.vrb THEN RETURN END;
    SO.Init( b, s );
    SO.WrStr( b, 'Close: opn ' );
    SO.WrLngCard( b, opn, 1 );
    SO.WrStr( b, ' cnt ' );
    SO.WrLngCard( b, cnt, 1 );
    Message( s );
  END Show;

BEGIN
  IF src # srcNull THEN
    Show();
    FIO.Close( src );
  END;
  src := srcNull;

  fdt := 0;
  fmt := 0;
  ok := FALSE;
  Lib2.Fill( tel, 0 );
  len := 0;
  ref := 0;
  pos := 0;
  top := 0;
  cnt := 0;
  (* opn must be kept *)
END Close;


VAR
  oldDoneModule: PROC;

PROCEDURE DoneModule();
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;
BEGIN
  IF Config.data.vrb THEN
    SO.Init( b, s );
    SO.WrStr( b, 'DoneModule: opn ' );
    SO.WrLngCard( b, opn, 1 );
    Message( s );
  END;
  oldDoneModule();
END DoneModule;

PROCEDURE InitModule();
BEGIN
  src := srcNull;
  Close();
  opn := 0;

  Lib3.Terminate( DoneModule, oldDoneModule );
END InitModule;

BEGIN
  InitModule();
END Rdr.
